function [ loss ] = compute_loss_mppca( x, K, d, Pi, W, mu, sigmasq )
    
    n = size(x, 1);
    D = size(x, 2);

    loss = zeros(n, K);
    for k = 1:K
        loss(:, k) = (-log(Pi(k)) + D/2 * log(2*pi)) * ones(n, 1);
        
        Wk = squeeze(W(k, :, 1:d(k))); % D x d
        if (size(Wk, 1) ~= D)
            Wk = Wk';
        end
        muk = mu(k, :);
        Sigma = sigmasq(k) * eye(D) + Wk * Wk'; % D x D
        
        Sigma = min(Sigma, 1e10 * ones(D, D));
        Sigma = max(Sigma, -1e10 * ones(D, D));
        if (det(Sigma) == 0)
            Sigma = Sigma + 1e-6 * eye(D);
        end
        
        [U, S, V] = svd(Sigma);
        for i = 1:D
            loss(:, k) = loss(:, k) + (0.5 * log(S(i, i))) * ones(n, 1);
        end
        
        A = (x - muk(ones(n, 1), :)) * Sigma^(-1); % n x D
        B = x - muk(ones(n, 1), :); % n x D
        likeli = sum(A .* B, 2); % n x 1
        loss(:, k) = loss(:, k) + 0.5 * likeli;
        
    end
    
    loss = sum(min(loss, [], 2));
    
end

